
unit UJours;

interface

uses
{$IFDEF FLASHPASCAL}
  Flash8, Date;
{$ELSE}
  SysUtils;
{$ENDIF}

const
  INCONNU   =  0;
  DIMANCHE  =  1;
  LUNDI     =  2;
  MARDI     =  3;
  MERCREDI  =  4;
  JEUDI     =  5;
  VENDREDI  =  6;
  SAMEDI    =  7;
  PREMIER   =  1;
  JANVIER   =  1;
  FEVRIER   =  2;
  MARS      =  3;
  AVRIL     =  4;
  MAI       =  5;
  JUIN      =  6;
  JUILLET   =  7;
  AOUT      =  8;
  SEPTEMBRE =  9;
  OCTOBRE   = 10;
  NOVEMBRE  = 11;
  DECEMBRE  = 12;

  NOMS_DES_JOURS: array[INCONNU..SAMEDI]of string = (
    'inconnu',
    'dimanche',
    'lundi',
    'mardi',
    'mercredi',
    'jeudi',
    'vendredi',
    'samedi'
  );

  NOMS_DES_MOIS: array[JANVIER..DECEMBRE]of string = (
{$IFDEF FLASHPASCAL}  
    'janvier',
    "fvrier",
    'mars',
    'avril',
    'mai',
    'juin',
    'juillet',
    "aot",
    'septembre',
    'octobre',
    'novembre',
    "dcembre"
{$ELSE}
    'janvier',
    'fvrier',
    'mars',
    'avril',
    'mai',
    'juin',
    'juillet',
    'aot',
    'septembre',
    'octobre',
    'novembre',
    'dcembre'
{$ENDIF}
  );

type
  TJour = record
    s, j, m, a: integer;
  end;

function Aujourdhui: TJour;
function ChaineJour(aJour: TJour): string;
function ChaineJourValide(s: string): boolean;
function JourChaine(s: string): TJour;
function Difference(aJour1, aJour2: TJour): integer;

implementation

const
  NOMBRE_JOURS_PAR_MOIS: array[JANVIER..DECEMBRE, 0..1]of integer = (
    (31, 31),
    (28, 29),
    (31, 31),
    (30, 30),
    (31, 31),
    (30, 30),
    (31, 31),
    (31, 31),
    (30, 30),
    (31, 31),
    (30, 30),
    (31, 31)
  );

function Bissext(aAnnee: integer): boolean;
begin
  result := (aAnnee mod 4  = 0) and not (
    (aAnnee mod 100 = 0) and not (aAnnee mod 400 = 0)
  );
end;

{$IFDEF FLASHPASCAL}
function StrToInt(s: string): integer;
var
  i, l, m: integer;
begin
  result := 0;

  l := Length(s);
  if l = 0 then
    exit;
    
  i := 1;
  while i <= l do
  begin
    if Pos(s[i], '0123456789') = 0 then
      exit;
    Inc(i);
  end;
  
  m := 1;
  for i := l downto 1 do
  begin
    Inc(result, m * Pos(s[i], '123456789'));
    m := 10 * m;
  end;
end;
{$ENDIF}

function Aujourdhui: TJour;
{$IFDEF FLASHPASCAL}
var
  d: tDate;
begin
  d := tDate.Create;
  result.s := d.GetDay + DIMANCHE;
  result.j := d.GetDate;
  result.m := d.GetMonth + JANVIER;
  result.a := d.GetFullYear;
end;
{$ELSE}
var
  dt: tDateTime;
  y, m, d: word;
begin
  dt := Date;
  DecodeDate(dt, y, m, d);
  result.s := DayOfWeek(dt);
  result.j := d;
  result.m := m;
  result.a := y;
end;
{$ENDIF}

function ChaineJour(aJour: TJour): string;
begin
  if aJour.j = PREMIER then
    result := '1er'
  else
    result := IntToStr(aJour.j);
  
  if aJour.s = INCONNU then
    result := result + ' '
  else
    result := NOMS_DES_JOURS[aJour.s] + ' ' + result + ' ';
  
  result := result + NOMS_DES_MOIS[aJour.m] + ' ' + IntToStr(aJour.a);
end;

function ChaineJourValide(s: string): boolean;
begin
  result := FALSE;
  if (Length(s) = 10) then
    if (s[3] = '/') and (s[6] = '/')
    and (Pos(s[01], '0123456789') > 0)
    and (Pos(s[02], '0123456789') > 0)
    and (Pos(s[04], '0123456789') > 0)
    and (Pos(s[05], '0123456789') > 0)
    and (Pos(s[07], '0123456789') > 0)
    and (Pos(s[08], '0123456789') > 0)
    and (Pos(s[09], '0123456789') > 0)
    and (Pos(s[10], '0123456789') > 0) then
      result := TRUE;
end;

function JourChaine(s: string): TJour;
begin
  if ChaineJourValide(s) then
  begin
    result.s := INCONNU;
    result.j := StrToInt(Copy(s, 1, 2));
    result.m := StrToInt(Copy(s, 4, 2));
    result.a := StrToInt(Copy(s, 7, 4));
  end else
  begin
    result.s := INCONNU;
    result.j := INCONNU;
    result.m := INCONNU;
    result.a := INCONNU;
  end;
end;

function Difference(aJour1, aJour2: TJour): integer;
var
  u: integer;
  n: integer;
  i: integer;
  j: TJour;
begin
  result := 0;
  
  aJour1.s := INCONNU;
  
  if aJour1.a = aJour2.a then
  begin
    if aJour1.m = aJour2.m then
    begin
      if aJour1.j = aJour2.j then
      begin
        exit
      end else
        if aJour1.j < aJour2.j then
          u := +1
        else
          u := -1;
    end else
      if aJour1.m < aJour2.m then
        u := +1
      else
        u := -1;
  end else
    if aJour1.a < aJour2.a then
      u := +1
    else
      u := -1;
  
  if u = -1 then
  begin
    j.j := aJour2.j;
    j.m := aJour2.m;
    j.a := aJour2.a;
    aJour2.j := aJour1.j;
    aJour2.m := aJour1.m;
    aJour2.a := aJour1.a;
    aJour1.j := j.j;
    aJour1.m := j.m;
    aJour1.a := j.a;
  end;
  
  while (aJour2.a - aJour1.a > 1) do
  begin
    n := 365;
    
    if (aJour1.m <= FEVRIER) and ((aJour1.j < 29) or (aJour1.m = JANVIER)) then
      Inc(n, Ord(Bissext(aJour1.a)))
    else
      Inc(n, Ord(Bissext(aJour1.a + 1)));
    
    Inc(result, n);
    
    if (aJour1.j = 29) and (aJour1.m = FEVRIER) then
    begin
      aJour1.j := 1;
      aJour1.m := MARS;
    end;
    
    Inc(aJour1.a);
  end;
  
  repeat
    Inc(aJour1.j);
    
    if aJour1.j > NOMBRE_JOURS_PAR_MOIS[aJour1.m, Ord(Bissext(aJour1.a))] then
    begin
      aJour1.j := PREMIER;
      Inc(aJour1.m);
      if aJour1.m > DECEMBRE then
      begin
        aJour1.m := JANVIER;
        Inc(aJour1.a);
      end;
    end; 
    
    Inc(result);
  until (aJour1.j = aJour2.j) and (aJour1.m = aJour2.m) and (aJour1.a = aJour2.a);
  
  result := u * result;
end;

end.
